#!/bin/bash

sudo touch /opt/inseego/log/devicelog/status.log
sudo chmod 777 /opt/inseego/log/devicelog/status.log

LOG_FILE="/opt/inseego/log/devicelog/status.log"

# Function for logging
log() {
    logger -t script_log -p user.debug "$1"
    echo "$(date) - $1" >> "$LOG_FILE"
}

log "============================Deployment Status==============================="

# Function to check the container status
check_container_status() {
    container_name=$1
    log "Checking status of container $container_name."
    status=$(docker inspect --format='{{.State.Status}}' "$container_name" 2>/dev/null)
    if [ "$status" == "running" ]; then
        log "Container $container_name is running."
        echo "Container $container_name is running."
    else
        log "Error :: Container $container_name is not running or not found."
        echo "Error :: Container $container_name is not running or not found."
    fi
}

# Function to check disk usage
check_disk_usage() {
    log "Checking disk usage."
    disk_usage=$(df -h | awk '$NF=="/"{printf "Disk Usage: %s\n", $5}')
    log "$disk_usage"
    echo "$disk_usage"
}

# Function to check memory usage
check_memory_usage() {
    log "Checking memory usage."
    memory_usage=$(free -m | awk 'NR==2{printf "Memory Usage: %s/%sMB (%.2f%%)\n", $3,$2,$3*100/$2 }')
    log "$memory_usage"
    echo "$memory_usage"
}

# Function to check CPU usage
check_cpu_usage() {
    log "Checking CPU usage."
    cpu_usage=$(top -bn1 | grep load | awk '{printf "CPU Load: %.2f\n", $(NF-2)}')
    log "$cpu_usage"
    echo "$cpu_usage"
}

# Function to check network usage
check_network_usage() {
    log "Checking network usage."
    network_usage=$(netstat -s | grep -E 'total packets received|segments received|packets sent out|segments send out' | awk '{print $1,$2}')
    log "Network Usage:"
    log "$network_usage"
    echo "Network Usage:"
    echo "$network_usage"
}

# Function to check system uptime
check_system_uptime() {
    log "Checking system uptime."
    system_uptime=$(uptime | awk '{printf "System Uptime: %s\n", $3}')
    log "$system_uptime"
    echo "$system_uptime"
}

# Add container names that you want to check
containers=("backend_device_service" "backend_user_service" "cache" "celery_beat" "celery_w" "celery_w_data_usage" "celery_w_device_status" "celery_w_mail_sending" "celery_w_qos" "celery_w_tunnel" "celery_w_wireguard" "celery_w_zabbix" "frontend" "nginx" "shell_access" "timescale" "wireguard" "zabbix" "zabbixUI")

# Loop through each container and check its status
for container in "${containers[@]}"; do
    check_container_status "$container"
done

# Check disk usage
check_disk_usage

# Check memory usage
check_memory_usage

# Check CPU usage
check_cpu_usage

# Check network usage
check_network_usage

# Check system uptime
check_system_uptime


