#/bin/bash

# Create and set permissions for the upgrade log file
sudo touch /opt/inseego/log/devicelog/upgrade.log
sudo chmod 777 /opt/inseego/log/devicelog/upgrade.log

# Set log file path
LOG_FILE="/opt/inseego/log/devicelog/upgrade.log"

# Function for logging
log() {
    echo "$(date) - $1" >> $LOG_FILE
}

log "#################Upgrade Started#####################"

echo "EM deployed in Public or Private Cloud?
Choose 1 for Public and 2 for Private"
read flag

log "Input for Deployment $flag"


# Copy Compose from upgrade folder to EM_PATH
EM_PATH="/opt/inseego/bin/edge_manager"
NGINX_PATH="/opt/inseego/srv"
log "Copying compose file to $EM_PATH."
if [[ $flag -eq 1 ]]; then
    cd /home/"$USER"/upgrade/aws && sudo cp docker-compose.yaml "$EM_PATH"/docker-compose.yaml
    log "Copied docker-compose.yaml from AWS upgrade folder to $EM_PATH."
    echo "Copied docker-compose.yaml from AWS upgrade folder to $EM_PATH."
elif [[ $flag -eq 2 ]]; then
    cd /home/"$USER"/upgrade/esxi && sudo cp docker-compose.yaml "$EM_PATH"/docker-compose.yaml
    log "Copied docker-compose.yaml from ESXI upgrade folder to $EM_PATH."
    echo "Copied docker-compose.yaml from ESXI upgrade folder to $EM_PATH."
else
    log "Invalid flag provided. Exiting script."
    echo "Invalid flag provided. Exiting script."
    exit 1
fi

# Changing the Ownership of $EM_PATH directory
log "Changing ownership of $EM_PATH."
sudo chown "$USER":"$USER" "$EM_PATH"/* "$EM_PATH"/.env
cd "$NGINX_PATH" && sudo chown "$USER":"$USER" .
log "Ownership changed for $EM_PATH and $NGINX_PATH."

# AWS ECR Authentication for pulling docker images
log "Authenticating with AWS ECR."
aws ecr get-login-password --region us-west-2 | docker login --username AWS --password-stdin 836891784212.dkr.ecr.us-west-2.amazonaws.com > /dev/null
log "AWS ECR authenticated for pulling docker images."
echo "AWS ECR authenticated for pulling docker images."

log "Checking and updating .env file for EM_VERSION."
if ! grep -q 'EM_VERSION' "$EM_PATH"/.env; then
    sudo sed -i '2 a EM_VERSION=2023.18.1.1' "$EM_PATH"/.env
    log "Updated EM_VERSION in .env file."
else
    sudo sed -i "s/EM_VERSION=.*/EM_VERSION=2023.18.1.1/g" "$EM_PATH"/.env
    log "Replaced existing EM_VERSION in .env file."
fi

# Set other environment variables
env_vars=("LOGGING_LEVEL=DEBUG" "DEBIAN_REPO_URL=https://repo.inseego5g.net/debs/" "CURRENT_ENV=prod")
for var in "${env_vars[@]}"; do
    if ! grep -q "${var%=*}" "$EM_PATH"/.env; then
        sudo sed -i "$((${#env_vars[@]}+2)) a ${var}" "$EM_PATH"/.env
        log "Added ${var%=*} to .env file."
    fi
done

# Ensure the MTU and SaveConfig values are set in the wg0.conf file
if ! grep -q 'MTU' /opt/inseego/etc/wireguard-data/config/wg0.conf; then
    sudo sed -i '3 a MTU = 1280' /opt/inseego/etc/wireguard-data/config/wg0.conf
    sudo sed -i '4 a SaveConfig = true' /opt/inseego/etc/wireguard-data/config/wg0.conf
    log "Added MTU and SaveConfig to wg0.conf file."
# Restart wireguard service and check the docker container status
	log "Restarting wireguard service."
	docker restart wireguard

fi

# Upgrading the EM to the latest version
cd "$EM_PATH" && docker-compose up -d
sudo rm -rf /opt/2023.16.2.2/ > /dev/null 2>&1
if grep -q 'EM_VERSION=2023.18.1.1' "$EM_PATH"/.env; then
    docker rmi -f 836891784212.dkr.ecr.us-west-2.amazonaws.com/frontend:2023.16.2.2 836891784212.dkr.ecr.us-west-2.amazonaws.com/backend:2023.16.2.2 836891784212.dkr.ecr.us-west-2.amazonaws.com/frontend:2023.18.1.0 836891784212.dkr.ecr.us-west-2.amazonaws.com/backend:2023.18.1.0 > /dev/null 2>&1
    log "EM upgraded successfully."
    echo "EM upgraded successfully"
else
    log "EM upgrade failed."
    echo "EM upgrade failed."
fi

# Adding a delay of 1 minute
log "Adding a delay of 1 minute for all the services to boot up"
echo "Waiting for all the services to boot up"
sleep 60

log "Checking the docker container status after upgrading."
docker ps

docker ps -a | while read -r line
do
    log "$line"
done

# Fetching and logging container logs
log "Fetching and logging container logs."
containers=("backend_device_service" "backend_user_service" "cache" "celery_beat" "celery_w" "celery_w_data_usage" "celery_w_device_status" "celery_w_mail_sending" "celery_w_qos" "celery_w_tunnel" "celery_w_wireguard" "celery_w_zabbix" "frontend" "nginx" "shell_access" "timescale" "wireguard" "zabbix" "zabbixUI")
for container in "${containers[@]}"; do
    log "&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
	             $container service logs
	 &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&"
    docker logs "$container" >> "$LOG_FILE"
done

# Log script completion
log "#################Script Execution Done#####################"


